/*=============================================================================
	OpenGL code, keep seperate from windowing code.
opengl.c
	Opengl code
=============================================================================*/
#ifndef OPENGL_C
#define OPENGL_C
#define _CRT_SECURE_NO_DEPRECATE
#include "include.h"

unsigned getTimeStamp(void);
void resize(int, int);
void initializeGL(int, int);
void drawScene(HWND, HDC);
void updatePos(POINT);
void drawText(const char *, float, float);
void drawGround(float y);
void drawErrors(void);

GLTFrame	camera;

// Called by WM_CREATE, functions can always assume current matrix is modelview and leave as current matrix upon exiting.
void initializeGL(int width, int height)
{
	double		aspect;
	GLfloat		lightAmbient[]	= {0.125f, 0.125f, 0.125f, 1.0f},
			lightDiffuse[]	= {0.25f, 0.25f, 0.25f, 1.0f},
			lightSpecular[]	= {0.8f, 0.8f, 0.5f, 1.0f},
			lightPos[]	= {0.0f, 2.0f, 1.0f, 1.0f};

	gltInitFrame(&camera);

	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
	glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
	glEnable(GL_LIGHT0);

	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glMaterialfv(GL_FRONT, GL_SPECULAR, lightSpecular);
	glMateriali(GL_FRONT, GL_SHININESS, 128);

	glClearColor(0.75f, 0.75f, 0.75f, 1.0f);
	glClearAccum(0.75f, 0.75f, 0.75f, 1.0f);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glShadeModel(GL_SMOOTH);

}

// Called by WMU_IDLE
void drawScene(HWND hwnd, HDC hdc)
{
	unsigned	frameStartTime, frameEndTime, frameElaspedTime;
	char		fps[8], buffer[80];
static unsigned framecount = 0, passcount = 0;	// hacky stuff doesnt get indented
	int i;

static unsigned temp;

temp = framecount;

frameStartTime = timeGetTime();
for(i=0;i<=10;i++)
{
	glMatrixMode(GL_MODELVIEW);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glPushMatrix();
		gltApplyCameraTransform(&camera);
		glEnable(GL_LIGHTING);
		glColor3f(1.0f, 1.0f, 1.0f);
		drawGround(-2.0f);
		drawGround(2.0f);
		glPushMatrix();
			glTranslatef(0.0f, 0.0f, -2.5f);
			
			glPushMatrix();
				glRotatef(framecount * 1.0f, 0.0f, 1.0f, 0.0f);
				glTranslatef(1.0f, 0.0f, 0.0f);
				glColor3f(0.0f, 1.0f, 0.0f);
				glutSolidSphere(0.1f, 32, 32);
			glPopMatrix();
			
			glRotatef(framecount * -1.0f, 0.0f, 1.0f, 0.0f);
			glColor3f(1.0f, 0.0f, 0.0f);
			gltDrawTorus(0.35, 0.15, 40, 40);
		glPopMatrix();
		glDisable(GL_LIGHTING);
	glPopMatrix();
	framecount++;
	glAccum(GL_ACCUM, 1.0f / (float)(11 - i) );
}
framecount = temp + 11;
	glMatrixMode(GL_MODELVIEW);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glPushMatrix();
		gltApplyCameraTransform(&camera);
		glEnable(GL_LIGHTING);
		glColor3f(1.0f, 1.0f, 1.0f);
		drawGround(-2.0f);
		drawGround(2.0f);
		glPushMatrix();
			glTranslatef(0.0f, 0.0f, -2.5f);
			
			glPushMatrix();
				glRotatef(framecount * 1.0f, 0.0f, 1.0f, 0.0f);
				glTranslatef(1.0f, 0.0f, 0.0f);
				glColor3f(0.0f, 1.0f, 0.0f);
				glutSolidSphere(0.1f, 32, 32);
			glPopMatrix();
			
			glRotatef(framecount * -1.0f, 0.0f, 1.0f, 0.0f);
			glColor3f(1.0f, 0.0f, 0.0f);
			gltDrawTorus(0.35, 0.15, 40, 40);
		glPopMatrix();
		glDisable(GL_LIGHTING);
	glPopMatrix();
	framecount++;
	glAccum(GL_RETURN, 0.25f);
	glClear( GL_ACCUM_BUFFER_BIT );


	Sleep(1);
	frameEndTime = timeGetTime();
	frameElaspedTime = frameEndTime - frameStartTime;
	wsprintf(fps, "%ufps", 1000 / frameElaspedTime);
	glColor3f( 1.0f, 1.0f, 1.0f );
	drawText(fps, 0.9f, 0.9f);
	glColor3f( 0.0f, 0.0f, 0.0f );
	sprintf(buffer, "Camera Frame");
	drawText(buffer, 0.05f, 0.25f);
	sprintf(buffer, "Forward : %1.3f %1.3f %1.3f", camera.vForward[0], camera.vForward[1], camera.vForward[2]);
	drawText(buffer, 0.1f, 0.1f);
	sprintf(buffer, "Position : %1.3f %1.3f %1.3f", camera.vLocation[0], camera.vLocation[1], camera.vLocation[2]);
	drawText(buffer, 0.1f, 0.15f);
	sprintf(buffer, "Up          : %1.3f %1.3f %1.3f", camera.vUp[0], camera.vUp[1], camera.vUp[2]);
	drawText(buffer, 0.1f, 0.2f);
	drawErrors();
	glFinish();
	SwapBuffers(hdc);
}

void updatePos(POINT mousePos)
{

	if (GetAsyncKeyState(VK_UP))
	{
		gltMoveFrameForward(&camera, 0.1f);
	}
	else if (GetAsyncKeyState(VK_DOWN))
	{
		gltMoveFrameForward(&camera, -0.1f);
	}
	else if (GetAsyncKeyState(VK_LEFT))
	{
		gltMoveFrameRight(&camera, 0.1f);
	}
	else if (GetAsyncKeyState(VK_RIGHT))
	{
		gltMoveFrameRight(&camera, -0.1f);
	}

	if (GetAsyncKeyState(VK_RETURN))
	{
		gltMoveFrameUp(&camera, 0.1f);
	}
	else if (GetAsyncKeyState(VK_SHIFT))
	{
		gltMoveFrameUp(&camera, -0.1f);
	}


	if (GetAsyncKeyState(VK_DELETE))
	{
		gltRotateFrameLocalY(&camera, 0.1f);
	}
	else if (GetAsyncKeyState(VK_NEXT))
	{
		gltRotateFrameLocalY(&camera, -0.1f);
	}
	else if (GetAsyncKeyState(VK_HOME))
	{
		gltRotateFrameLocalZ(&camera, 0.1f); // doesnt work...
	}
	else if (GetAsyncKeyState(VK_END))
	{
		gltRotateFrameLocalZ(&camera, -0.1f);
	}

	if (mousePos.x > 0)
	{
		gltRotateFrameLocalY(&camera,  mousePos.x * -0.001f);
	}
	else if (mousePos.x < 0)
	{
		gltRotateFrameLocalY(&camera, -mousePos.x * 0.001f);
	}
	else if (mousePos.y > 0)
	{
		gltRotateFrameLocalY(&camera, mousePos.y * 0.001f);
	}
	else if (mousePos.y < 0)
	{
		gltRotateFrameLocalY(&camera, mousePos.y * -0.001f);
	}
}

void resize( int width, int height )
{
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (!height)
		height = 1;
	gluPerspective( 45.0, (double) width / height, 0.1, 50.0 );
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

// Could ease use by adding printf like format strings
void drawText(const char *string, float x, float y)
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
		glLoadIdentity();
		glOrtho (0.0f, 1.0f, 0.0f, 1.0f, 1.0f, 0.0f);
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
			glLoadIdentity();
			glRasterPos2f(x, y);
			glListBase(1000);
			glCallLists(lstrlen(string), GL_UNSIGNED_BYTE, string);	
		glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}

void drawErrors(void)
{
	GLenum	errCode;
	int	errCount = 0;

	while (1)
	{
		errCode = glGetError();
		if (errCode != GL_NO_ERROR)
		{
			errCount++;
			drawText(gluErrorString(errCode), 0.5f, 0.5f + (errCount * 0.05f) );
		} else {
			break;
		}
	}
}

void drawGround(float y)
{
	GLfloat fExtent = 20.0f;
	GLfloat fStep = 1.0f;
	GLint iLine;

	glBegin(GL_LINES);
	for(iLine = -fExtent; iLine <= fExtent; iLine += fStep)
	{
		glVertex3f(iLine, y, fExtent);    // Draw Z lines
		glVertex3f(iLine, y, -fExtent);
		glVertex3f(fExtent, y, iLine);
		glVertex3f(-fExtent, y, iLine);
	}
	glEnd();
}
#endif
